#include <stdio.h>
#include <string.h>
#include "reseau.h"

time_t dt;
static int asource,adest;
static char *femission, *freception;

void prendre_de_la_couche_reseau(int *AS, int *AD, ttampon *T)
/* Lecture des donnes sur stdin, on quitte s'il n'y a plus rien  lire */
{
  if (fgets(T->info,MAXTAILLETAMPON,stdin)==NULL)
  {
    printf("Plus de donnees a envoyer\n");
    exit(0);
  }
  T->taille=strlen(T->info);
  *AS=asource;
  *AD=adest;
  return;
}

void construire_trame(int AS, int AD, ttampon T, ttrame *Trame)
/* Construit la trame  partir du tampon et des adresses, et insre le CRC */
{
  Trame->AS=AS;
  Trame->AD=AD;
  Trame->tampon=T;
  Trame->CRC=calcule_CRC(*Trame);
  return;
}

void envoyer_vers_couche_physique(ttrame Trame)
/* Envoie la trame sur le support physique */
{
  FILE *f;
  while ((f=fopen(femission,"wb"))==NULL);
  fwrite(&Trame,sizeof(ttrame),1,f);
  fclose(f);
  return;
}

void prendre_de_la_couche_physique(ttrame *Trame)
/* Rcupre la trame sur le support physique */
{
  FILE *f;
  while ((f=fopen(freception,"rb"))==NULL);
  fread(Trame,sizeof(ttrame),1,f);
  fclose(f);
  remove(freception);
  return;
}

void recuperer_donnees(ttrame Trame, ttampon *T, int *AS, int *AD)
/* Extraire le tampon et les adresses de la trame */
{
  *T=Trame.tampon;
  *AS=Trame.AS;
  *AD=Trame.AD;
  return;
}

void envoyer_vers_couche_reseau(ttampon T, int AS, int AD)
/* Affichage des donnes sur stdout */
{
  int i;
  for (i=0; i<T.taille; i++)  printf("%c", T.info[i]);
  fflush(stdout);
  return;
}

void attendre(int signal)
/* Attend un ou des signaux */
{
  while (!signale(signal));
  return;
}

char signale(int signal)
/* Renvoie 1 si un des signaux passs en paramtre est mis
   SIGRECU  : une trame sans erreurs qui nous est destine est arrive
   SIGTEMPO : timeout
   SIGEMIS  : il reste des donnes  envoyer */
{
  time_t aux;
  FILE *fic;
  ttrame Trame;
  if (signal & SIGRECU)
    if ((fic=fopen(freception,"rb"))!=NULL)
    {
      fread(&Trame,sizeof(ttrame),1,fic);
      fclose(fic);
      if ((Trame.AD==asource) && (calcule_CRC(Trame)==Trame.CRC)) return(1);
    }
  if (signal & SIGTEMPO)
  {
    aux=time(NULL);
    if (difftime(aux,dt)>=TIMEOUT) return(1);
  }
  if (signal & SIGEMIS)
    if (!feof(stdin)) return(1);
  return(0);
}

char calcule_CRC(ttrame Trame)
/* Calcul du CRC de la trame, le CRC est ici un caractre de parit */
{
  int i;
  char resultat=0;
  char *p=(char *)&Trame;
  for (i=0; i<sizeof(ttrame)-sizeof(char); i++)
    resultat=(resultat & !*(p+i)) | (!resultat & *(p+i));
  return(resultat);
}

void tempo(void)
/* Dclenche le temporisateur */
{
  dt=time(NULL);
  return;
}

void initialise_couche_physique(char *emission, char *reception)
/* Initialise la couche physique, les paramtres sont les fichiers d'change */
{
  femission=emission;
  freception=reception;
  remove(femission);
  remove(freception);
  return;
}

void initialise_couche_reseau(int source, int destination)
/* Initialise la couche rseau, les paramtres sont les adresses de la connection */
{
  asource=source;
  adest=destination;
  return;
}
